@@ -5,10 +5,13 @@ import skia
 from .document import RecordingDocument
 from .errors import DrawbotError
 from .gstate import GraphicsState, GraphicsStateMixin
+from .shaping import alignGlyphPositions
+
+
+DEFAULT_CANVAS_DIMENSIONS = (1000, 1000)
 
 
 class Drawing:
-
     def __init__(self, document=None, flipCanvas=True):
         self._flipCanvas = flipCanvas
         self._reset(document)
@@ -24,7 +27,7 @@ class Drawing:
     @property
     def _canvas(self):
         if self._skia_canvas is None:
-            self.size(1000, 1000)  # This will create the canvas
+            self.size(*DEFAULT_CANVAS_DIMENSIONS)  # This will create the canvas
         return self._skia_canvas
 
     @_canvas.setter
@@ -35,14 +38,30 @@ class Drawing:
         self._reset()
 
     def endDrawing(self):
-        ...
+        self._canvas = None
+        self._document.endDrawing()
 
     def size(self, width, height):
         if self._document.isDrawing:
-            raise DrawbotError("size() can't be called if there's already a canvas active")
-        self.newPage(width,  height)
+            raise DrawbotError(
+                "size() can't be called if there's already a canvas active"
+            )
+        self.newPage(width, height)
 
-    def newPage(self, width, height):
+    def newPage(self, width=None, height=None):
+        if (width is not None and height is None) or (
+            height is not None and width is None
+        ):
+            raise TypeError(
+                "newPage() takes either no argument or two - width and height"
+            )
+        if width is None and height is None:
+            width = getattr(self._document, "pageWidth", None)
+            height = getattr(self._document, "pageHeight", None)
+            if width is None or height is None:
+                # this is because dimensions are set only after drawing starts,
+                # if you use this function before, it breaks
+                width, height = DEFAULT_CANVAS_DIMENSIONS
         if self._document.isDrawing:
             self._document.endPage()
             self._gstate = GraphicsState()
@@ -73,6 +92,7 @@ class Drawing:
 
     def polygon(self, firstPoint, *points, close=True):
         from .path import BezierPath
+
         bez = BezierPath()
         bez.polygon(firstPoint, *points, close=close)
         self.drawPath(bez)
@@ -96,8 +116,9 @@ class Drawing:
             # Hard Skia crash otherwise
             return
 
-        glyphsInfo = self._gstate.textStyle.shape(txt)
-        blob = self._gstate.textStyle.makeTextBlob(glyphsInfo, align)
+        textStyle = self._gstate.textStyle
+        glyphsInfo = textStyle.shape(txt)
+        alignGlyphPositions(glyphsInfo, align)
 
         x, y = position
 
@@ -105,7 +126,33 @@ class Drawing:
             self._canvas.translate(x, y)
             if self._flipCanvas:
                 self._canvas.scale(1, -1)
-            self._drawItem(self._canvas.drawTextBlob, blob, 0, 0)
+            if "COLR" not in textStyle.ttFont:
+                builder = skia.TextBlobBuilder()
+                builder.allocRunPos(
+                    textStyle.skFont, glyphsInfo.gids, glyphsInfo.positions
+                )
+                blob = builder.make()
+                self._drawItem(self._canvas.drawTextBlob, blob, 0, 0)
+            else:
+                from blackrenderer.backends.skia import SkiaCanvas
+
+                ttFont = textStyle.ttFont
+                brFont = textStyle.brFont
+                if textStyle.variations:
+                    brFont.setLocation(textStyle.variations)
+
+                canvas = SkiaCanvas(self._canvas)
+                scaleFactor = textStyle.fontSize / brFont.unitsPerEm
+                a, r, g, b = (ch / 255 for ch in self._gstate.fillPaint.color)
+                textColor = (r, g, b, a)
+                for gid, (x, y) in zip(glyphsInfo.gids, glyphsInfo.positions):
+                    glyphName = ttFont.getGlyphName(gid)
+                    with self._savedCanvasState():
+                        self._canvas.translate(x, y)
+                        self._canvas.scale(scaleFactor, -scaleFactor)
+                        brFont.drawGlyph(
+                            glyphName, canvas, palette=None, textColor=textColor
+                        )
 
     def image(self, imagePath, position, alpha=1.0):
         im = self._getImage(imagePath)
@@ -168,15 +215,19 @@ class Drawing:
     def savedState(self):
         self._stack.append(self._gstate.copy())
         self._canvas.save()
-        yield
-        self._canvas.restore()
-        self._gstate = self._stack.pop()
+        try:
+            yield
+        finally:
+            self._canvas.restore()
+            self._gstate = self._stack.pop()
 
     @contextlib.contextmanager
     def _savedCanvasState(self):
         self._canvas.save()
-        yield
-        self._canvas.restore()
+        try:
+            yield
+        finally:
+            self._canvas.restore()
 
     def saveImage(self, fileName, **kwargs):
         if self._document.isDrawing:
@@ -208,6 +259,7 @@ def _makeWrapper(name):
     def wrapper(self, *args, **kwargs):
         method = getattr(self._gstate, name)
         return method(*args, **kwargs)
+
     wrapper.__qualname__ = f"Drawing.{name}"
     return wrapper
 
