@@ -1,10 +1,11 @@
 import logging
 import os
 import skia
+import uharfbuzz as hb
 from .errors import DrawbotError
-from .font import makeTTFontFromSkiaTypeface, tagToInt
+from .font import makeHBFaceFromSkiaTypeface, makeTTFontFromSkiaTypeface, tagToInt
 from .segmenting import textSegments, reorderedSegments
-from .shaping import getShapeFuncForSkiaTypeface
+from .shaping import shape
 
 
 class cached_property(object):
@@ -15,7 +16,7 @@ class cached_property(object):
     with an ordinary attribute. Deleting the attribute resets the property."""
 
     def __init__(self, func):
-        self.__doc__ = getattr(func, '__doc__')
+        self.__doc__ = getattr(func, "__doc__")
         self.func = func
 
     def __get__(self, obj, cls):
@@ -34,14 +35,18 @@ class GraphicsStateMixin:
         if color is None:
             self.fillPaint = self.fillPaint.copy(somethingToDraw=False, shader=None)
         else:
-            self.fillPaint = self.fillPaint.copy(color=color, somethingToDraw=True, shader=None)
+            self.fillPaint = self.fillPaint.copy(
+                color=color, somethingToDraw=True, shader=None
+            )
 
     def stroke(self, *args):
         color = _colorArgs(args)
         if color is None:
             self.strokePaint = self.strokePaint.copy(somethingToDraw=False, shader=None)
         else:
-            self.strokePaint = self.strokePaint.copy(color=color, somethingToDraw=True, shader=None)
+            self.strokePaint = self.strokePaint.copy(
+                color=color, somethingToDraw=True, shader=None
+            )
 
     def blendMode(self, blendMode):
         if blendMode not in _blendModes:
@@ -65,7 +70,9 @@ class GraphicsStateMixin:
     def lineDash(self, firstValue=None, *values):
         if firstValue is None:
             if values:
-                raise TypeError("lineDash() argument(s) should be None, or one or more numbers")
+                raise TypeError(
+                    "lineDash() argument(s) should be None, or one or more numbers"
+                )
         if firstValue is None:
             assert not values
             self.strokePaint = self.strokePaint.copy(lineDash=None)
@@ -89,9 +96,19 @@ class GraphicsStateMixin:
             colors=colors,
             positions=locations,
         )
-        self.fillPaint = self.fillPaint.copy(shader=shader, fill=None, somethingToDraw=True)
+        self.fillPaint = self.fillPaint.copy(
+            shader=shader, fill=None, somethingToDraw=True
+        )
 
-    def radialGradient(self, startPoint, endPoint=None, colors=None, locations=None, startRadius=0, endRadius=100):
+    def radialGradient(
+        self,
+        startPoint,
+        endPoint=None,
+        colors=None,
+        locations=None,
+        startRadius=0,
+        endRadius=100,
+    ):
         # MakeRadial(
         #   center: skia.Point,
         #   radius: float,
@@ -101,9 +118,13 @@ class GraphicsStateMixin:
         #   flags: int = 0,
         #   localMatrix: skia.Matrix = None) → skia.Shader
         if startRadius != 0:
-            logging.warning("radialGradient: startRadius != 0 ignored (it's not supported in drawbot-skia)")
+            logging.warning(
+                "radialGradient: startRadius != 0 ignored (it's not supported in drawbot-skia)"
+            )
         if endPoint is not None and endPoint != startPoint:
-            logging.warning("radialGradient: endPoint argument ignored (it's not supported in drawbot-skia)")
+            logging.warning(
+                "radialGradient: endPoint argument ignored (it's not supported in drawbot-skia)"
+            )
         colors = [_colorTupleToInt(_colorArgs(c)) for c in colors]
         shader = skia.GradientShader.MakeRadial(
             center=startPoint,
@@ -111,7 +132,9 @@ class GraphicsStateMixin:
             colors=colors,
             positions=locations,
         )
-        self.fillPaint = self.fillPaint.copy(shader=shader, fill=None, somethingToDraw=True)
+        self.fillPaint = self.fillPaint.copy(
+            shader=shader, fill=None, somethingToDraw=True
+        )
 
     def shadow(self, offset, blur=None, color=None):
         if offset is None:
@@ -132,6 +155,9 @@ class GraphicsStateMixin:
     def fontSize(self, size):
         self.textStyle = self.textStyle.copy(fontSize=size)
 
+    def lineHeight(self, value):
+        self.textStyle = self.textStyle.copy(lineHeight=value)
+
     def openTypeFeatures(self, *, resetFeatures=False, **features):
         if resetFeatures:
             currentFeatures = {}
@@ -222,7 +248,6 @@ class GraphicsState(GraphicsStateMixin):
 
 
 class _ImmutableContainer:
-
     def __init__(self, **properties):
         self.__dict__.update(properties)
         self._names = set(properties)
@@ -374,67 +399,34 @@ class TextStyle(_ImmutableContainer):
     variations = {}  # won't get mutated
     language = None
     font = None
+    lineHeight = None
 
     def __init__(self, **properties):
         super().__init__(**properties)
 
+    @cached_property
+    def fontObjects(self):
+        return _getFontObjects(self.font)
+
     @cached_property
     def skFont(self):
-        typeface, ttFont = self._getTypefaceAndTTFont(self.font)
+        typeface = self.fontObjects.skTypeface
+        ttFont = self.fontObjects.ttFont
         if self.variations and "fvar" in ttFont:
-            typeface = self._cloneTypeface(typeface, ttFont, self.variations)
-        font = self._makeFontFromTypeface(typeface, self.fontSize)
-        return font
+            typeface = _cloneTypeface(typeface, ttFont, self.variations)
+        return _makeFontFromTypeface(typeface, self.fontSize)
 
     @property
     def ttFont(self):
-        _, ttFont = self._getTypefaceAndTTFont(self.font)
-        return ttFont
+        return self.fontObjects.ttFont
 
-    @staticmethod
-    def _getTypefaceAndTTFont(fontNameOrPath):
-        cacheKey = fontNameOrPath
-        if cacheKey not in _fontCache:
-            if fontNameOrPath is None:
-                typeface = skia.Typeface(None)
-            else:
-                fontNameOrPath = os.fspath(fontNameOrPath)
-                if not os.path.exists(fontNameOrPath):
-                    typeface = skia.Typeface(fontNameOrPath)
-                else:
-                    typeface = skia.Typeface.MakeFromFile(fontNameOrPath)
-                    if typeface is None:
-                        raise DrawbotError(f"can't load font: {fontNameOrPath}")
-            ttFont = makeTTFontFromSkiaTypeface(typeface)
-            _fontCache[cacheKey] = typeface, ttFont
-        return _fontCache[cacheKey]
+    @property
+    def hbFont(self):
+        return self.fontObjects.hbFont
 
-    @staticmethod
-    def _makeFontFromTypeface(typeface, size):
-        font = skia.Font(typeface, size)
-        font.setForceAutoHinting(False)
-        font.setHinting(skia.FontHinting.kNone)
-        font.setSubpixel(True)
-        font.setEdging(skia.Font.Edging.kAntiAlias)
-        return font
-
-    @staticmethod
-    def _cloneTypeface(typeface, ttFont, variations):
-        fvar = ttFont.get("fvar")
-        defaultLocation = {a.axisTag: variations.get(a.axisTag, a.defaultValue) for a in fvar.axes}
-        tags = [a.axisTag for a in fvar.axes]
-        location = [(tag, variations.get(tag, defaultLocation[tag])) for tag in tags]
-        makeCoord = skia.FontArguments.VariationPosition.Coordinate
-        rawCoords = [makeCoord(tagToInt(tag), value) for tag, value in location]
-        coords = skia.FontArguments.VariationPosition.Coordinates(rawCoords)
-        pos = skia.FontArguments.VariationPosition(coords)
-        fontArgs = skia.FontArguments()
-        fontArgs.setVariationDesignPosition(pos)
-        return typeface.makeClone(fontArgs)
-
-    @cached_property
-    def _shapeFunc(self):
-        return getShapeFuncForSkiaTypeface(self.skFont.getTypeface())
+    @property
+    def brFont(self):
+        return self.fontObjects.brFont
 
     def shape(self, txt):
         segments, baseLevel = textSegments(txt)
@@ -442,7 +434,8 @@ class TextStyle(_ImmutableContainer):
         startPos = (0, 0)
         glyphsInfo = None
         for runChars, script, bidiLevel, index in segments:
-            runInfo = self._shapeFunc(
+            runInfo = shape(
+                self.hbFont,
                 runChars,
                 fontSize=self.skFont.getSize(),
                 startPos=startPos,
@@ -463,32 +456,85 @@ class TextStyle(_ImmutableContainer):
         glyphsInfo.baseLevel = baseLevel
         return glyphsInfo
 
-    def alignGlyphPositions(self, glyphsInfo, align):
-        textWidth = glyphsInfo.endPos[0]
-        if align is None:
-            align = "left" if not glyphsInfo.baseLevel else "right"
-        xOffset = 0
-        if align == "right":
-            xOffset = -textWidth
-        elif align == "center":
-            xOffset = -textWidth / 2
-        glyphsInfo.positions = [(x + xOffset, y) for x, y in glyphsInfo.positions]
-
-    def makeTextBlob(self, glyphsInfo, align):
-        self.alignGlyphPositions(glyphsInfo, align)
-        builder = skia.TextBlobBuilder()
-        builder.allocRunPos(self.skFont, glyphsInfo.gids, glyphsInfo.positions)
-        return builder.make()
+    def getLineHeight(self):
+        if self.lineHeight is not None:
+            return self.lineHeight
+        else:
+            return self.fontSize * 1.2
 
 
-# Font cache dict
-# - keys: font name or path string
-# - values: (skTypeface, ttFont) tuples
-_fontCache = {}
+_fontObjectsCache = {}
 
 
 def clearFontCache():
-    _fontCache.clear()
+    _fontObjectsCache.clear()
+
+
+def _getFontObjects(fontNameOrPath):
+    fontObjects = _fontObjectsCache.get(fontNameOrPath)
+    if fontObjects is None:
+        fontObjects = FontObjects(fontNameOrPath)
+        _fontObjectsCache[fontNameOrPath] = fontObjects
+    return fontObjects
+
+
+class FontObjects:
+    def __init__(self, fontNameOrPath):
+        self.fontNameOrPath = fontNameOrPath
+
+    @cached_property
+    def skTypeface(self):
+        fontNameOrPath = self.fontNameOrPath
+        if fontNameOrPath is None:
+            typeface = skia.Typeface(None)
+        else:
+            fontNameOrPath = os.fspath(fontNameOrPath)
+            if not os.path.exists(fontNameOrPath):
+                typeface = skia.Typeface(fontNameOrPath)
+            else:
+                typeface = skia.Typeface.MakeFromFile(fontNameOrPath)
+                if typeface is None:
+                    raise DrawbotError(f"can't load font: {fontNameOrPath}")
+        return typeface
+
+    @cached_property
+    def ttFont(self):
+        return makeTTFontFromSkiaTypeface(self.skTypeface)
+
+    @cached_property
+    def hbFont(self):
+        return hb.Font(makeHBFaceFromSkiaTypeface(self.skTypeface))
+
+    @cached_property
+    def brFont(self):
+        from blackrenderer.font import BlackRendererFont
+
+        return BlackRendererFont(ttFont=self.ttFont, hbFont=self.hbFont)
+
+
+def _cloneTypeface(typeface, ttFont, variations):
+    fvar = ttFont.get("fvar")
+    defaultLocation = {
+        a.axisTag: variations.get(a.axisTag, a.defaultValue) for a in fvar.axes
+    }
+    tags = [a.axisTag for a in fvar.axes]
+    location = [(tag, variations.get(tag, defaultLocation[tag])) for tag in tags]
+    makeCoord = skia.FontArguments.VariationPosition.Coordinate
+    rawCoords = [makeCoord(tagToInt(tag), value) for tag, value in location]
+    coords = skia.FontArguments.VariationPosition.Coordinates(rawCoords)
+    pos = skia.FontArguments.VariationPosition(coords)
+    fontArgs = skia.FontArguments()
+    fontArgs.setVariationDesignPosition(pos)
+    return typeface.makeClone(fontArgs)
+
+
+def _makeFontFromTypeface(typeface, size):
+    font = skia.Font(typeface, size)
+    font.setForceAutoHinting(False)
+    font.setHinting(skia.FontHinting.kNone)
+    font.setSubpixel(True)
+    font.setEdging(skia.Font.Edging.kAntiAlias)
+    return font
 
 
 def _colorTupleToInt(color):
@@ -521,34 +567,34 @@ def _colorArgs(args):
 
 
 _blendModesList = [
-    'normal',
-    'multiply',
-    'screen',
-    'overlay',
-    'darken',
-    'lighten',
-    'colorDodge',
-    'colorBurn',
-    'softLight',
-    'hardLight',
-    'difference',
-    'exclusion',
-    'hue',
-    'saturation',
-    'color',
-    'luminosity',
-    'clear',
-    'copy',
-    'sourceIn',
-    'sourceOut',
-    'sourceAtop',
-    'destinationOver',
-    'destinationIn',
-    'destinationOut',
-    'destinationAtop',
-    'xOR',
-    'plusDarker',
-    'plusLighter',
+    "normal",
+    "multiply",
+    "screen",
+    "overlay",
+    "darken",
+    "lighten",
+    "colorDodge",
+    "colorBurn",
+    "softLight",
+    "hardLight",
+    "difference",
+    "exclusion",
+    "hue",
+    "saturation",
+    "color",
+    "luminosity",
+    "clear",
+    "copy",
+    "sourceIn",
+    "sourceOut",
+    "sourceAtop",
+    "destinationOver",
+    "destinationIn",
+    "destinationOut",
+    "destinationAtop",
+    "xOR",
+    "plusDarker",
+    "plusLighter",
 ]
 
 _blendModes = set(_blendModesList)
