@@ -5,14 +5,20 @@ from unicodedata2 import category
 # Monkeypatch bidi to use unicodedata2
 import unicodedata2
 import bidi.algorithm
+
 bidi.algorithm.bidirectional = unicodedata2.bidirectional
 bidi.algorithm.category = unicodedata2.category
 bidi.algorithm.mirrored = unicodedata2.mirrored
 from bidi.algorithm import (  # noqa: ignore E402
-    get_empty_storage, get_base_level, get_embedding_levels,
-    explicit_embed_and_overrides, resolve_weak_types,
-    resolve_neutral_types, resolve_implicit_levels,
-    reorder_resolved_levels, PARAGRAPH_LEVELS,
+    get_empty_storage,
+    get_base_level,
+    get_embedding_levels,
+    explicit_embed_and_overrides,
+    resolve_weak_types,
+    resolve_neutral_types,
+    resolve_implicit_levels,
+    reorder_resolved_levels,
+    PARAGRAPH_LEVELS,
 )
 from bidi.mirror import MIRRORED  # noqa: ignore E402
 
@@ -25,10 +31,10 @@ def textSegments(txt):
     storage = getBiDiInfo(txt)
 
     levels = [None] * len(txt)
-    for ch in storage['chars']:
-        levels[ch['index']] = ch['level']
+    for ch in storage["chars"]:
+        levels[ch["index"]] = ch["level"]
 
-    prevLevel = storage['base_level']
+    prevLevel = storage["base_level"]
     for i, level in enumerate(levels):
         if level is None:
             levels[i] = prevLevel
@@ -50,7 +56,7 @@ def textSegments(txt):
         script, bidiLevel = segment[0]
         segments.append((runChars, script, bidiLevel, index))
         index = nextIndex
-    return segments, storage['base_level']
+    return segments, storage["base_level"]
 
 
 def reorderedSegments(segments, isRTL, isSegmentRTLFunc):
@@ -73,7 +79,7 @@ def detectScript(txt):
         scr = charScript[i]
         if scr in UNKNOWN_SCRIPT:
             if i:
-                scr = charScript[i-1]
+                scr = charScript[i - 1]
             else:
                 scr = None
             cat = category(ch)
@@ -105,6 +111,7 @@ def detectScript(txt):
 
 # copied from bidi/algorthm.py and modified to be more useful for us.
 
+
 def getBiDiInfo(text, *, upper_is_rtl=False, base_dir=None, debug=False):
     """
     Set `upper_is_rtl` to True to treat upper case chars as strong 'R'
@@ -124,8 +131,8 @@ def getBiDiInfo(text, *, upper_is_rtl=False, base_dir=None, debug=False):
     else:
         base_level = PARAGRAPH_LEVELS[base_dir]
 
-    storage['base_level'] = base_level
-    storage['base_dir'] = ('L', 'R')[base_level]
+    storage["base_level"] = base_level
+    storage["base_dir"] = ("L", "R")[base_level]
 
     get_embedding_levels(text, storage, upper_is_rtl, debug)
     assert len(text) == len(storage["chars"])
