@@ -1,7 +1,7 @@
 size(256, 256)
 
 scale(0.5, 0.5)
-p = '../images/drawbot.png'
+p = "../images/drawbot.png"
 
 blendMode("multiply")
 image(p, (0, 0))
